@@ -1,10 +1,9 @@
 /*
- * Copyright 2003-2007 Gentoo Foundation
+ * Copyright 2003-2012 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.56 2007/08/20 09:54:15 vapier Exp $
  *
- * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #include "paxinc.h"
@@ -14,17 +13,19 @@
  * binary defines into readable strings.
  */
 #define QUERY(n) { #n, n }
-typedef struct {
+typedef const struct {
 	const char *str;
-	int value;
+	/* We use unsigned int as we assume it's at least 32 bits.  This covers
+	   all our uses so far as they have been limited to that size.  */
+	unsigned int value;
 } pairtype;
-static inline const char *find_pairtype(pairtype *pt, int type)
+static inline const char *find_pairtype(pairtype *pt, unsigned int type)
 {
-	int i;
+	size_t i;
 	for (i = 0; pt[i].str; ++i)
 		if (type == pt[i].value)
 			return pt[i].str;
-	return "UNKNOWN TYPE";
+	return "UNKNOWN_TYPE";
 }
 
 /* translate misc elf EI_ defines */
@@ -32,20 +33,17 @@ static pairtype elf_ei_class[] = {
 	QUERY(ELFCLASSNONE),
 	QUERY(ELFCLASS32),
 	QUERY(ELFCLASS64),
-	QUERY(ELFCLASSNUM),
 	{ 0, 0 }
 };
 static pairtype elf_ei_data[] = {
 	QUERY(ELFDATANONE),
 	QUERY(ELFDATA2LSB),
 	QUERY(ELFDATA2MSB),
-	QUERY(ELFDATANUM),
 	{ 0, 0 }
 };
 static pairtype elf_ei_version[] = {
 	QUERY(EV_NONE),
 	QUERY(EV_CURRENT),
-	QUERY(EV_NUM),
 	{ 0, 0 }
 };
 static pairtype elf_ei_osabi[] = {
@@ -53,6 +51,7 @@ static pairtype elf_ei_osabi[] = {
 	QUERY(ELFOSABI_SYSV),
 	QUERY(ELFOSABI_HPUX),
 	QUERY(ELFOSABI_NETBSD),
+	QUERY(ELFOSABI_GNU),
 	QUERY(ELFOSABI_LINUX),
 	QUERY(ELFOSABI_SOLARIS),
 	QUERY(ELFOSABI_AIX),
@@ -61,6 +60,7 @@ static pairtype elf_ei_osabi[] = {
 	QUERY(ELFOSABI_TRU64),
 	QUERY(ELFOSABI_MODESTO),
 	QUERY(ELFOSABI_OPENBSD),
+	QUERY(ELFOSABI_ARM_AEABI),
 	QUERY(ELFOSABI_ARM),
 	QUERY(ELFOSABI_STANDALONE),
 	{ 0, 0 }
@@ -73,7 +73,7 @@ const char *get_elfeitype(int ei_type, int type)
 		case EI_VERSION: return find_pairtype(elf_ei_version, type);
 		case EI_OSABI:   return find_pairtype(elf_ei_osabi, type);
 	}
-	return "UNKNOWN EI TYPE";
+	return "UNKNOWN_EI_TYPE";
 }
 
 /* translate elf ET_ defines */
@@ -83,36 +83,73 @@ static pairtype elf_etypes[] = {
 	QUERY(ET_EXEC),
 	QUERY(ET_DYN),
 	QUERY(ET_CORE),
-	QUERY(ET_NUM),
-	QUERY(ET_LOOS),
-	QUERY(ET_HIOS),
-	QUERY(ET_LOPROC),
-	QUERY(ET_HIPROC),
 	{ 0, 0 }
 };
 
-int get_etype(elfobj *elf)
+unsigned int get_etype(const elfobj *elf)
 {
-	int type;
 	if (elf->elf_class == ELFCLASS32)
-		type = EGET(EHDR32(elf->ehdr)->e_type);
+		return EGET(EHDR32(elf->ehdr)->e_type);
 	else
-		type = EGET(EHDR64(elf->ehdr)->e_type);
-	return type;
+		return EGET(EHDR64(elf->ehdr)->e_type);
 }
 
-const char *get_elfetype(elfobj *elf)
+const char *get_elfetype(const elfobj *elf)
 {
 	return find_pairtype(elf_etypes, get_etype(elf));
 }
 
-const char *get_endian(elfobj *elf)
+const char *get_endian(const elfobj *elf)
 {
-	if (elf->data[EI_DATA] == ELFDATA2LSB)
-		return (char *) "LE";
-	if (elf->data[EI_DATA] == ELFDATA2MSB)
-		return (char *) "BE";
-	return (char *) "??";
+	switch (elf->data[EI_DATA]) {
+		case ELFDATA2LSB: return "LE";
+		case ELFDATA2MSB: return "BE";
+		default:          return "??";
+	}
+}
+
+/* translate elf EF_ defines -- tricky as it's based on EM_ */
+static unsigned int get_eflags(const elfobj *elf)
+{
+	if (elf->elf_class == ELFCLASS32)
+		return EGET(EHDR32(elf->ehdr)->e_flags);
+	else
+		return EGET(EHDR64(elf->ehdr)->e_flags);
+}
+
+static int arm_eabi_poker(const elfobj *elf)
+{
+	unsigned int emachine, eflags;
+
+	if (ELFOSABI_NONE != elf->data[EI_OSABI])
+		return -1;
+
+	emachine = get_emtype(elf);
+	eflags = get_eflags(elf);
+
+	if (emachine == EM_ARM)
+		return EF_ARM_EABI_VERSION(eflags) >> 24;
+	else
+		return -1;
+}
+
+const char *get_elf_eabi(const elfobj *elf)
+{
+	static char buf[26];
+	int eabi = arm_eabi_poker(elf);
+	if (eabi >= 0)
+		snprintf(buf, sizeof(buf), "%i", eabi);
+	else
+		strcpy(buf, "?");
+	return buf;
+}
+
+const char *get_elfosabi(const elfobj *elf)
+{
+	const char *str = get_elfeitype(EI_OSABI, elf->data[EI_OSABI]);
+	if (strncmp(str, "ELFOSABI_", 9) == 0)
+		str += 9;
+	return str;
 }
 
 void print_etypes(FILE *stream)
@@ -129,10 +166,10 @@ void print_etypes(FILE *stream)
 		fprintf(stream, "\n");
 }
 
-int etype_lookup(const char *str)
+unsigned int etype_lookup(const char *str)
 {
 	if (*str == 'E') {
-		int i;
+		size_t i;
 		for (i = 0; elf_etypes[i].str; ++i) {
 			if (strcmp(str, elf_etypes[i].str) == 0)
 				return elf_etypes[i].value;
@@ -141,7 +178,6 @@ int etype_lookup(const char *str)
 	return atoi(str);
 }
 
-
 /* translate elf EM_ defines */
 static pairtype elf_emtypes[] = {
 	QUERY(EM_NONE),
@@ -234,27 +270,28 @@ static pairtype elf_emtypes[] = {
 	QUERY(EM_SEP),
 	QUERY(EM_ARCA),
 	QUERY(EM_UNICORE),
-	QUERY(EM_NUM),
+	QUERY(EM_ALTERA_NIOS2),
+	QUERY(EM_AARCH64),
+	QUERY(EM_TILEPRO),
+	QUERY(EM_MICROBLAZE),
+	QUERY(EM_TILEGX),
 	QUERY(EM_ALPHA),
 	{ 0, 0 }
 };
 
-int get_emtype(elfobj *elf)
+unsigned int get_emtype(const elfobj *elf)
 {
-	int type;
 	if (elf->elf_class == ELFCLASS32)
-		type = EGET(EHDR32(elf->ehdr)->e_machine);
+		return EGET(EHDR32(elf->ehdr)->e_machine);
 	else
-		type = EGET(EHDR64(elf->ehdr)->e_machine);
-	return type;
+		return EGET(EHDR64(elf->ehdr)->e_machine);
 }
 
-const char *get_elfemtype(elfobj *elf)
+const char *get_elfemtype(const elfobj *elf)
 {
 	return find_pairtype(elf_emtypes, get_emtype(elf));
 }
 
-
 /* translate elf PT_ defines */
 static pairtype elf_ptypes[] = {
 	QUERY(PT_NULL),
@@ -265,7 +302,6 @@ static pairtype elf_ptypes[] = {
 	QUERY(PT_SHLIB),
 	QUERY(PT_PHDR),
 	QUERY(PT_TLS),
-	QUERY(PT_NUM),
 	QUERY(PT_GNU_EH_FRAME),
 	QUERY(PT_GNU_STACK),
 	QUERY(PT_GNU_RELRO),
@@ -313,7 +349,34 @@ static pairtype elf_dtypes[] = {
 	QUERY(DT_ENCODING),
 	QUERY(DT_PREINIT_ARRAY),
 	QUERY(DT_PREINIT_ARRAYSZ),
-	QUERY(DT_NUM),
+	QUERY(DT_GNU_PRELINKED),
+	QUERY(DT_GNU_CONFLICTSZ),
+	QUERY(DT_GNU_LIBLISTSZ),
+	QUERY(DT_CHECKSUM),
+	QUERY(DT_PLTPADSZ),
+	QUERY(DT_MOVEENT),
+	QUERY(DT_MOVESZ),
+	QUERY(DT_GNU_HASH),
+	QUERY(DT_TLSDESC_PLT),
+	QUERY(DT_TLSDESC_GOT),
+	QUERY(DT_GNU_CONFLICT),
+	QUERY(DT_GNU_LIBLIST),
+	QUERY(DT_CONFIG),
+	QUERY(DT_DEPAUDIT),
+	QUERY(DT_AUDIT),
+	QUERY(DT_PLTPAD),
+	QUERY(DT_MOVETAB),
+	QUERY(DT_SYMINFO),
+	QUERY(DT_VERSYM),
+	QUERY(DT_RELACOUNT),
+	QUERY(DT_RELCOUNT),
+	QUERY(DT_FLAGS_1),
+	QUERY(DT_VERDEF),
+	QUERY(DT_VERDEFNUM),
+	QUERY(DT_VERNEED),
+	QUERY(DT_VERNEEDNUM),
+	QUERY(DT_AUXILIARY),
+	QUERY(DT_FILTER),
 	{ 0, 0 }
 };
 const char *get_elfdtype(int type)
@@ -340,23 +403,16 @@ static pairtype elf_shttypes[] = {
 	QUERY(SHT_PREINIT_ARRAY),
 	QUERY(SHT_GROUP),
 	QUERY(SHT_SYMTAB_SHNDX),
-	QUERY(SHT_NUM),
-	QUERY(SHT_LOOS),
+	QUERY(SHT_GNU_ATTRIBUTES),
+	QUERY(SHT_GNU_HASH),
 	QUERY(SHT_GNU_LIBLIST),
 	QUERY(SHT_CHECKSUM),
-	QUERY(SHT_LOSUNW),
 	QUERY(SHT_SUNW_move),
 	QUERY(SHT_SUNW_COMDAT),
 	QUERY(SHT_SUNW_syminfo),
 	QUERY(SHT_GNU_verdef),
 	QUERY(SHT_GNU_verneed),
 	QUERY(SHT_GNU_versym),
-	QUERY(SHT_HISUNW),
-	QUERY(SHT_HIOS),
-	QUERY(SHT_LOPROC),
-	QUERY(SHT_HIPROC),
-	QUERY(SHT_LOUSER),
-	QUERY(SHT_HIUSER),
 	{ 0, 0 }
 };
 const char *get_elfshttype(int type)
@@ -371,18 +427,125 @@ static pairtype elf_stttypes[] = {
 	QUERY(STT_FUNC),
 	QUERY(STT_SECTION),
 	QUERY(STT_FILE),
-	QUERY(STT_LOPROC),
-	QUERY(STT_HIPROC),
-	QUERY(STB_LOCAL),
-	QUERY(STB_GLOBAL),
-	QUERY(STB_WEAK),
-	QUERY(STB_LOPROC),
-	QUERY(STB_HIPROC),
+	QUERY(STT_COMMON),
+	QUERY(STT_TLS),
+	QUERY(STT_GNU_IFUNC),
 	{ 0, 0 }
 };
 const char *get_elfstttype(int type)
 {
-	return find_pairtype(elf_stttypes, type & 0xF);
+	return find_pairtype(elf_stttypes, type);
+}
+
+/* translate elf STB_ defines */
+static pairtype elf_stbtypes[] = {
+	QUERY(STB_LOCAL),
+	QUERY(STB_GLOBAL),
+	QUERY(STB_WEAK),
+	QUERY(STB_GNU_UNIQUE),
+	{ 0, 0 }
+};
+const char *get_elfstbtype(int type)
+{
+	return find_pairtype(elf_stbtypes, type);
+}
+
+/* translate elf STV_ defines */
+static pairtype elf_stvtypes[] = {
+	QUERY(STV_DEFAULT),
+	QUERY(STV_INTERNAL),
+	QUERY(STV_HIDDEN),
+	QUERY(STV_PROTECTED),
+	{ 0, 0 }
+};
+const char *get_elfstvtype(int type)
+{
+	return find_pairtype(elf_stvtypes, type);
+}
+
+/* translate elf SHN_ defines */
+static pairtype elf_shntypes[] = {
+	QUERY(SHN_UNDEF),
+	QUERY(SHN_BEFORE),
+	QUERY(SHN_AFTER),
+	QUERY(SHN_ABS),
+	QUERY(SHN_COMMON),
+	QUERY(SHN_XINDEX),
+	{ 0, 0 }
+};
+const char *get_elfshntype(int type)
+{
+	if (type && type < SHN_LORESERVE)
+		return "DEFINED";
+	return find_pairtype(elf_shntypes, type);
+}
+
+/* translate elf NT_ defines */
+static pairtype elf_nttypes_GNU[] = {
+	QUERY(NT_GNU_ABI_TAG),
+	QUERY(NT_GNU_HWCAP),
+	QUERY(NT_GNU_BUILD_ID),
+	QUERY(NT_GNU_GOLD_VERSION),
+	{ 0, 0 }
+};
+static pairtype elf_nttypes_core[] = {
+	QUERY(NT_PRSTATUS),
+	QUERY(NT_FPREGSET),
+	QUERY(NT_PRPSINFO),
+	QUERY(NT_PRXREG),
+	QUERY(NT_TASKSTRUCT),
+	QUERY(NT_PLATFORM),
+	QUERY(NT_AUXV),
+	QUERY(NT_GWINDOWS),
+	QUERY(NT_ASRS),
+	QUERY(NT_PSTATUS),
+	QUERY(NT_PSINFO),
+	QUERY(NT_PRCRED),
+	QUERY(NT_UTSNAME),
+	QUERY(NT_LWPSTATUS),
+	QUERY(NT_LWPSINFO),
+	QUERY(NT_PRFPXREG),
+	QUERY(NT_SIGINFO),
+	QUERY(NT_FILE),
+	QUERY(NT_PRXFPREG),
+	QUERY(NT_PPC_VMX),
+	QUERY(NT_PPC_SPE),
+	QUERY(NT_PPC_VSX),
+	QUERY(NT_386_TLS),
+	QUERY(NT_386_IOPERM),
+	QUERY(NT_X86_XSTATE),
+	QUERY(NT_S390_HIGH_GPRS),
+	QUERY(NT_S390_TIMER),
+	QUERY(NT_S390_TODCMP),
+	QUERY(NT_S390_TODPREG),
+	QUERY(NT_S390_CTRS),
+	QUERY(NT_S390_PREFIX),
+	QUERY(NT_S390_LAST_BREAK),
+	QUERY(NT_S390_SYSTEM_CALL),
+	QUERY(NT_S390_TDB),
+	QUERY(NT_ARM_VFP),
+	QUERY(NT_ARM_TLS),
+	QUERY(NT_ARM_HW_BREAK),
+	QUERY(NT_ARM_HW_WATCH),
+	{ 0, 0 }
+};
+static pairtype elf_nttypes_fallback[] = {
+	QUERY(NT_VERSION),
+	{ 0, 0 }
+};
+const char *get_elfnttype(uint16_t e_type, const char *name, int type)
+{
+	if (name) {
+		if (!strcmp(name, "GNU"))
+			return find_pairtype(elf_nttypes_GNU, type);
+
+		/* Unknown extension, so just fallback to common ones. */
+	}
+
+	if (e_type == ET_CORE)
+		return find_pairtype(elf_nttypes_core, type);
+	else
+		return find_pairtype(elf_nttypes_fallback, type);
 }
 
 /* Read an ELF into memory */
@@ -395,7 +558,7 @@ const char *get_elfstttype(int type)
 	((buff[EI_CLASS] == ELFCLASS32 || buff[EI_CLASS] == ELFCLASS64) && \
 	 (buff[EI_DATA] == ELFDATA2LSB || buff[EI_DATA] == ELFDATA2MSB) && \
 	 (buff[EI_VERSION] == EV_CURRENT))
-elfobj *readelf_buffer(const char *filename, char *buffer, size_t buffer_len)
+elfobj *readelf_buffer(const char *filename, const void *buffer, size_t buffer_len)
 {
 	elfobj *elf;
 
@@ -403,10 +566,7 @@ elfobj *readelf_buffer(const char *filename, char *buffer, size_t buffer_len)
 	if (buffer == NULL || buffer_len < EI_NIDENT)
 		return NULL;
 
-	elf = xmalloc(sizeof(*elf));
-	if (elf == NULL)
-		return NULL;
-	memset(elf, 0x00, sizeof(*elf));
+	elf = xzalloc(sizeof(*elf));
 
 	elf->fd = -1;
 	elf->len = buffer_len;
@@ -439,12 +599,24 @@ free_elf_and_return:
 		elf->base_filename = elf->base_filename + 1;
 	elf->elf_class = elf->data[EI_CLASS];
 	do_reverse_endian = (ELF_DATA != elf->data[EI_DATA]);
-	elf->ehdr = (void*)elf->data;
+
+	/* for arches that need alignment, we have to make sure the buffer
+	 * is strictly aligned.  archive (.a) files only align to 2 bytes
+	 * while the arch can easily require 8.  so dupe the buffer so
+	 * that our local copy is always aligned (since we can't shift the
+	 * file mapping back and forth a few bytes).
+	 */
+	if (!__PAX_UNALIGNED_OK && ((unsigned long)elf->vdata & 0x7)) {
+		elf->_data = xmalloc(elf->len);
+		memcpy(elf->_data, elf->data, elf->len);
+		elf->data = elf->_data;
+		elf->data_end = elf->_data + elf->len;
+	}
 
 #define READELF_HEADER(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
 		char invalid; \
-		Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+		const Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
 		Elf ## B ## _Off size; \
 		/* verify program header */ \
 		invalid = 0; \
@@ -453,7 +625,7 @@ free_elf_and_return:
 		else if (EGET(ehdr->e_phentsize) != sizeof(Elf ## B ## _Phdr)) \
 			invalid = 3; \
 		else { \
-			elf->phdr = elf->data + EGET(ehdr->e_phoff); \
+			elf->phdr = elf->vdata + EGET(ehdr->e_phoff); \
 			size = EGET(ehdr->e_phnum) * EGET(ehdr->e_phentsize); \
 			if (elf->phdr < elf->ehdr || /* check overflow */ \
 			    elf->phdr + size < elf->phdr || /* before start of mem */ \
@@ -471,7 +643,7 @@ free_elf_and_return:
 		else if (EGET(ehdr->e_shentsize) != sizeof(Elf ## B ## _Shdr)) \
 			invalid = 3; \
 		else { \
-			elf->shdr = elf->data + EGET(ehdr->e_shoff); \
+			elf->shdr = elf->vdata + EGET(ehdr->e_shoff); \
 			size = EGET(ehdr->e_shnum) * EGET(ehdr->e_shentsize); \
 			if (elf->shdr < elf->ehdr || /* check overflow */ \
 			    elf->shdr + size < elf->shdr || /* before start of mem */ \
@@ -503,8 +675,8 @@ elfobj *_readelf_fd(const char *filename, int fd, size_t len, int read_only)
 			return NULL;
 	}
 
-	buffer = (char*)mmap(0, len, PROT_READ | (read_only ? 0 : PROT_WRITE), (read_only ? MAP_PRIVATE : MAP_SHARED), fd, 0);
-	if (buffer == (char*)MAP_FAILED) {
+	buffer = mmap(0, len, PROT_READ | (read_only ? 0 : PROT_WRITE), (read_only ? MAP_PRIVATE : MAP_SHARED), fd, 0);
+	if (buffer == MAP_FAILED) {
 		warn("mmap on '%s' of %li bytes failed :(", filename, (unsigned long)len);
 		return NULL;
 	}
@@ -525,19 +697,19 @@ elfobj *_readelf(const char *filename, int read_only)
 	struct stat st;
 	int fd;
 
-	if (stat(filename, &st) == -1)
-		return NULL;
-
 	if ((fd = open(filename, (read_only ? O_RDONLY : O_RDWR))) == -1)
 		return NULL;
 
-	/* make sure we have enough bytes to scan e_ident */
-	if (st.st_size <= EI_NIDENT) {
-close_fd_and_return:
+	if (fstat(fd, &st) == -1) {
+ close_fd_and_return:
 		close(fd);
 		return NULL;
 	}
 
+	/* make sure we have enough bytes to scan e_ident */
+	if (st.st_size <= EI_NIDENT)
+		goto close_fd_and_return;
+
 	ret = readelf_fd(filename, fd, st.st_size);
 	if (ret == NULL)
 		goto close_fd_and_return;
@@ -548,12 +720,13 @@ close_fd_and_return:
 /* undo the readelf() stuff */
 void unreadelf(elfobj *elf)
 {
-	if (elf->is_mmap) munmap(elf->data, elf->len);
+	if (elf->is_mmap) munmap((void *)elf->vdata, elf->len);
 	if (elf->fd != -1) close(elf->fd);
+	if (!__PAX_UNALIGNED_OK) free(elf->_data);
 	free(elf);
 }
 
-char *pax_short_hf_flags(unsigned long flags)
+const char *pax_short_hf_flags(unsigned long flags)
 {
 	static char buffer[7];
 
@@ -573,7 +746,7 @@ char *pax_short_hf_flags(unsigned long flags)
  * lower case: explicitly disabled
  * upper case: explicitly enabled
  *      -    : default */
-char *pax_short_pf_flags(unsigned long flags)
+const char *pax_short_pf_flags(unsigned long flags)
 {
 	static char buffer[7];
 
@@ -594,62 +767,12 @@ char *pax_short_pf_flags(unsigned long flags)
 	    ((flags & PF_RANDEXEC) && (flags & PF_NORANDEXEC)) || \
 	    ((flags & PF_EMUTRAMP) && (flags & PF_NOEMUTRAMP)) || \
 	    ((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)))
-		warn("inconsistent state detected.  flags=%lX\n", flags);
+		warn("inconsistent state detected.  flags=%lX", flags);
 
 	return buffer;
 }
 
-unsigned long pax_pf2hf_flags(unsigned long paxflags)
-{
-	unsigned long flags = 0;
-	char *pf_flags = pax_short_pf_flags(paxflags);
-	size_t x, len = strlen(pf_flags);
-	for (x = 0; x < len; x++) {
-		switch (pf_flags[x]) {
-			case 'p':
-				flags |= HF_PAX_PAGEEXEC;
-				break;
-			case 'P':
-				flags = (flags & ~HF_PAX_PAGEEXEC) | HF_PAX_SEGMEXEC;
-				break;
-			case 'E':
-				flags |= HF_PAX_EMUTRAMP;
-				break;
-			case 'e':
-				flags = (flags & ~HF_PAX_EMUTRAMP);
-				break;
-			case 'm':
-				flags |= HF_PAX_MPROTECT;
-				break;
-			case 'M':
-				flags = (flags & ~HF_PAX_MPROTECT);
-				break;
-			case 'r':
-				flags |= HF_PAX_RANDMMAP;
-				break;
-			case 'R':
-				flags = (flags & ~HF_PAX_RANDMMAP);
-				break;
-			case 'X':
-				flags |= HF_PAX_RANDEXEC;
-				break;
-			case 'x':
-				flags = (flags & ~HF_PAX_RANDEXEC);
-				break;
-			case 's':
-				flags |= HF_PAX_SEGMEXEC;
-				break;
-			case 'S':
-				flags = (flags & ~HF_PAX_SEGMEXEC) | HF_PAX_PAGEEXEC;
-				break;
-			default:
-				break;
-		}
-	}
-	return flags;
-}
-
-char *gnu_short_stack_flags(unsigned long flags)
+const char *gnu_short_stack_flags(unsigned long flags)
 {
 	static char buffer[4];
 
@@ -661,32 +784,31 @@ char *gnu_short_stack_flags(unsigned long flags)
 	return buffer;
 }
 
-void *elf_findsecbyname(elfobj *elf, const char *name)
+const void *elf_findsecbyname(const elfobj *elf, const char *name)
 {
 	unsigned int i;
-	char *shdr_name;
-	void *ret = NULL;
+	const char *shdr_name;
+	const void *ret = NULL;
 
 	if (elf->shdr == NULL) return NULL;
 
 #define FINDSEC(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
-	Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
-	Elf ## B ## _Shdr *shdr = SHDR ## B (elf->shdr); \
-	Elf ## B ## _Shdr *strtbl; \
+	const Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
+	const Elf ## B ## _Shdr *shdr = SHDR ## B (elf->shdr); \
+	const Elf ## B ## _Shdr *strtbl; \
 	Elf ## B ## _Off offset; \
 	uint16_t shstrndx = EGET(ehdr->e_shstrndx); \
 	uint16_t shnum = EGET(ehdr->e_shnum); \
 	if (shstrndx >= shnum) return NULL; \
 	strtbl = &(shdr[shstrndx]); \
 	for (i = 0; i < shnum; ++i) { \
-		if (EGET(shdr[i].sh_offset) >= elf->len - EGET(ehdr->e_shentsize)) continue; \
 		offset = EGET(strtbl->sh_offset) + EGET(shdr[i].sh_name); \
 		if (offset >= (Elf ## B ## _Off)elf->len) continue; \
-		shdr_name = (char*)(elf->data + offset); \
+		shdr_name = elf->data + offset; \
 		if (!strcmp(shdr_name, name)) { \
 			if (ret) warnf("Multiple '%s' sections !?", name); \
-			ret = (void*)&(shdr[i]); \
+			ret = &shdr[i]; \
 		} \
 	} }
 	FINDSEC(32)
@@ -694,50 +816,3 @@ void *elf_findsecbyname(elfobj *elf, const char *name)
 
 	return ret;
 }
-
-int elf_max_pt_load(elfobj *elf)
-{
-#define MAX_PT_LOAD(B) \
-	if (elf->elf_class == ELFCLASS ## B) { \
-	Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
-	switch (EGET(ehdr->e_ident[EI_OSABI])) { \
-	case ELFOSABI_NONE: \
-	case ELFOSABI_NETBSD: \
-	case ELFOSABI_FREEBSD: \
-	case ELFOSABI_LINUX: \
-	case ELFOSABI_ARM:     return 2; \
-	case ELFOSABI_OPENBSD: return 7; \
-	} }
-	MAX_PT_LOAD(32)
-	MAX_PT_LOAD(64)
-
-	return 1;
-}
-#if 0
- # define ELFOSABI_NONE           0       /* UNIX System V ABI */
- # define ELFOSABI_SYSV           0       /* Alias.  */
- # define ELFOSABI_HPUX           1       /* HP-UX */
- # define ELFOSABI_NETBSD         2       /* NetBSD.  */
- # define ELFOSABI_LINUX          3       /* Linux.  */
- # define ELFOSABI_SOLARIS        6       /* Sun Solaris.  */
- # define ELFOSABI_AIX            7       /* IBM AIX.  */
- # define ELFOSABI_IRIX           8       /* SGI Irix.  */
- # define ELFOSABI_FREEBSD        9       /* FreeBSD.  */
- # define ELFOSABI_TRU64          10      /* Compaq TRU64 UNIX.  */
- # define ELFOSABI_MODESTO        11      /* Novell Modesto.  */
- # define ELFOSABI_OPENBSD        12      /* OpenBSD.  */
- # define ELFOSABI_ARM            97      /* ARM */
- # define ELFOSABI_STANDALONE     255     /* Standalone (embedded) application */
-
- /* These 3 ABIs should be in elf.h but are not.
-  * http://www.caldera.com/developers/gabi/latest/ch4.eheader.html#generic_osabi_values
-  */
-
- # define ELFOSABI_OPENVMS 13     /* OpenVMS */
- # define ELFOSABI_NSK     14     /* Hewlett-Packard Non-Stop Kernel */
- # define ELFOSABI_AROS    15     /* Amiga Research OS */
-
- #4 reserved for IA32 GNU Mach/Hurd
- #5 reserved for 86Open common IA32 ABI
-
-#endif
