@@ -1,10 +1,9 @@
 /*
- * Copyright 2005-2007 Gentoo Foundation
+ * Copyright 2005-2012 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.31 2007/05/23 22:27:27 solar Exp $
  *
- * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2012 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2012 Mike Frysinger  - <vapier@gentoo.org>
  *
  * Make sure all of the common elf stuff is setup as we expect
  */
@@ -12,33 +11,48 @@
 #ifndef _PORTING_H
 #define _PORTING_H
 
-#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(*arr))
-#undef __PAX_UTILS_CLEANUP
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
+#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(*arr))
+
+#include <assert.h>
+#include <ctype.h>
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <fnmatch.h>
+#include <getopt.h>
+#include <inttypes.h>
+#include <libgen.h>
+#include <limits.h>
+#include <pwd.h>
+#include <regex.h>
+#include <sched.h>
+#include <signal.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <limits.h>
-#include <fcntl.h>
-#include <assert.h>
-#include <getopt.h>
-#include <libgen.h>
-#include <ctype.h>
-#include <pwd.h>
-#include <fnmatch.h>
-#include <regex.h>
-
 #include <sys/mman.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #include "elf.h"
+#if !defined(__FreeBSD__) && !defined(__OpenBSD__)
+# include <alloca.h>
+#endif
 #if defined(__linux__)
-# include <endian.h>
+# include <sys/prctl.h>
+# if !defined(HAVE_CONFIG_H) || defined(HAVE_LINUX_SECUREBITS_H)
+#  include <linux/securebits.h>
+# endif
+#endif
+#if defined(__GLIBC__) || defined(__UCLIBC__) || defined(__ANDROID__)
 # include <byteswap.h>
+# include <endian.h>
 #elif defined(__FreeBSD__)
 # include <sys/endian.h>
 #elif defined(__sun__)
@@ -60,12 +74,35 @@
 # define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG ""
 #endif
 
+#undef PAX_UTILS_CLEANUP
 /* bounds checking code will fart on free(NULL) even though that
  * is valid usage.  So let's wrap it if need be.
  */
 #ifdef __BOUNDS_CHECKING_ON
 # define free(ptr) do { if (ptr) free(ptr); } while (0)
-# define __PAX_UTILS_CLEANUP
+# define PAX_UTILS_CLEANUP 1
+#endif
+/* LSAN (Leak Sanitizer) will complain about things we leak. */
+#ifdef __SANITIZE_ADDRESS__
+# define PAX_UTILS_CLEANUP 1
+#endif
+/* Coverity catches some things we leak on purpose. */
+#ifdef __COVERITY__
+# define PAX_UTILS_CLEANUP 1
+#endif
+#ifndef PAX_UTILS_CLEANUP
+# define PAX_UTILS_CLEANUP 0
+#endif
+
+/* Few arches can safely do unaligned accesses */
+#if defined(__cris__) || \
+    defined(__i386__) || \
+    defined(__powerpc__) || \
+    defined(__s390__) || \
+    defined(__x86_64__)
+# define __PAX_UNALIGNED_OK 1
+#else
+# define __PAX_UNALIGNED_OK 0
 #endif
 
 #if !defined(bswap_16)
@@ -106,9 +143,18 @@
 # endif
 #endif
 
-#if !defined(_POSIX_PATH_MAX) && !defined(PATH_MAX)
+#define _minmax(x, y, op) \
+	({ __typeof__(x) __x = (x); __typeof__(y) __y = (y); (__x op __y ? __x : __y); })
+#if !defined(min)
+# define min(x, y) _minmax(x, y, <)
+#endif
+#if !defined(max)
+# define max(x, y) _minmax(x, y, >)
+#endif
+
+#if !defined(_POSIX_PATH_MAX) && !defined(PATH_MAX) /* __PAX_UTILS_PATH_MAX */
 # define __PAX_UTILS_PATH_MAX 8192
-#elif _POSIX_PATH_MAX > PATH_MAX
+#elif _POSIX_PATH_MAX > PATH_MAX /* __PAX_UTILS_PATH_MAX */
 # define __PAX_UTILS_PATH_MAX _POSIX_PATH_MAX
 #else
 # define __PAX_UTILS_PATH_MAX PATH_MAX
@@ -138,27 +184,13 @@
 #  define ELF_DATA ELFDATA2MSB
 # elif defined(_LITTLE_ENDIAN)
 #  define ELF_DATA ELFDATA2LSB
-# elif defined(_BIG_ENDIAN)
+# elif defined(_BIG_ENDIAN) || defined(__BIG_ENDIAN__)
 #  define ELF_DATA ELFDATA2MSB
 # else
 #  error "no idea what the native byte order is"
 # endif
 #endif
 
-/*
- * in case we are not defined by proper/up-to-date system headers,
- * we check for a whole lot of things and copy them from elf.h.
- */
-
-#ifndef PT_GNU_STACK
-# define PT_GNU_STACK	0x6474e551
-#endif
-
-/* not in <=binutils-2.14.90.0.8 (should come in by way of .9) */
-#ifndef PT_GNU_RELRO
-# define PT_GNU_RELRO	0x6474e552
-#endif
-
 /*
  * propably will never be official added to the toolchain.
  * But none the less we should try to get 0x65041580 reserved
@@ -189,4 +221,13 @@
 # endif
 #endif
 
+#ifndef O_CLOEXEC
+# define O_CLOEXEC 0
+#endif
+#ifndef O_PATH
+# define O_PATH 0
+#endif
+
+#define __unused__ __attribute__((__unused__))
+
 #endif /* _PORTING_H */
